local Recount2_Widgets = Recount2.Widgets

local CHECKED_TEXTURE = "Interface\\Addons\\Recount2\\Textures\\checkbox-true.tga"
local UNCHECKED_TEXTURE = "Interface\\Addons\\Recount2\\Textures\\checkbox-false.tga"

local bit_band = bit.band
local bit_bor = bit.bor
local bit_bnot = bit.bnot

local tinsert = table.insert

local updateFuncs = {}

local compositeValues = {}
local compositeFrames = {}

local listValues = {}
local listFrames = {}

local function SetFontSize(string, size)
	local Font, Height, Flags = string:GetFont()
	string:SetFont(Font, size, Flags)
end

local function GetValue(self)
	return self.value
end

local function SetValue(self, value)
	self.value = value

	if value then
		self.checkImage:SetTexture(CHECKED_TEXTURE)
	else
		self.checkImage:SetTexture(UNCHECKED_TEXTURE)
	end

	if self.compositeID then
		if value then
			compositeValues[self.compositeID] = bit_bor(self.flagValue, compositeValues[self.compositeID])
		else
			compositeValues[self.compositeID] = bit_band(self.flagNotValue, compositeValues[self.compositeID])
		end
		if updateFuncs[self.compositeID] then
			updateFunc = updateFuncs[self.compositeID]
			updateFunc[1](unpack(updateFunc[2]))
		end
	elseif self.listID then
		if value then
			listValues[self.listID][self.listValue] = true
		else
			listValues[self.listID][self.listValue] = nil
		end
		if updateFuncs[self.listID] then
			updateFunc = updateFuncs[self.listID]
			updateFunc[1](unpack(updateFunc[2]))
		end
	end	
end

local function OnEnter(self)
	self.background:Show()
end

local function OnLeave(self)
	self.background:Hide()
end

local function OnMouseDown(self)
	self:SetValue(not self.value)
end

local function SetupCheckbox(parent, width, height, text)
	local checkbox = CreateFrame("FRAME", nil, parent)

	checkbox:SetWidth(width)
	checkbox:SetHeight(height)

	checkbox.background = checkbox:CreateTexture(nil, "BACKGROUND")
	checkbox.background:SetAllPoints(checkbox)
	checkbox.background:SetTexture(1.0, 1.0, 0.0, 0.25)
	checkbox.background:Hide()

	checkbox.checkImage = checkbox:CreateTexture(nil, "OVERLAY")
	checkbox.checkImage:SetPoint("TOPLEFT", checkbox, "TOPLEFT")
	checkbox.checkImage:SetWidth(height)
	checkbox.checkImage:SetHeight(height)
	checkbox.checkImage:SetTexture(UNCHECKED_TEXTURE)

	checkbox.text = checkbox:CreateFontString(nil,"OVERLAY","GameFontHighlightSmall")
	checkbox.text:SetPoint("LEFT", checkbox, "LEFT", height+1, 0)
	checkbox.text:SetPoint("RIGHT", checkbox, "RIGHT")
	checkbox.text:SetJustifyH("LEFT")
	checkbox.text:SetText(text)
	checkbox.text:SetTextColor(1,1,1,1)
	Recount2:AddFontString(checkbox.text)
	SetFontSize(checkbox.text, height*0.8)

	checkbox.value = false

	checkbox.OnEnter = OnEnter
	checkbox.OnLeave = OnLeave
	checkbox.OnMouseDown = OnMouseDown
	checkbox.SetValue = SetValue

	checkbox:SetScript("OnEnter", checkbox.OnEnter)
	checkbox:SetScript("OnLeave", checkbox.OnLeave)
	checkbox:SetScript("OnMouseDown", checkbox.OnMouseDown)
	checkbox:EnableMouse()

	return checkbox
end

function Recount2_Widgets:CreateCheckbox(parent, width, height, text, compositeID, flagValue)
	local checkbox = SetupCheckbox(parent, width, height, text)
	
	checkbox.compositeID = compositeID
	checkbox.flagValue = flagValue
	if flagValue then
		checkbox.flagNotValue = bit_bnot(flagValue)
	end

	if compositeID then
		if compositeValues[compositeID]==nil then
			compositeValues[compositeID] = 0
			compositeFrames[compositeID] = {}
		end
		tinsert(compositeFrames[compositeID], checkbox)
	end

	return checkbox
end

function Recount2_Widgets:CreateCheckboxList(parent, width, height, text, listID, listValue)
	local checkbox = SetupCheckbox(parent, width, height, text)

	checkbox.listID = listID
	checkbox.listValue = listValue

	if listID then
		if listValues[listID]==nil then
			listValues[listID] = {}
			listFrames[listID] = {}
		end
		tinsert(listFrames[listID], checkbox)
	end

	return checkbox
end

function Recount2_Widgets:GetCompositeCheckValue(compositeID)
	return compositeValues[compositeID]
end

function Recount2_Widgets:SetCompositeCheckValue(compositeID, value)
	for _, checkbox in ipairs(compositeFrames[compositeID]) do
		if bit_band(checkbox.flagValue, value)==checkbox.flagValue then
			checkbox:SetValue(true)
		else
			checkbox:SetValue(false)
		end
	end
end

function Recount2_Widgets:GetListCheckValue(listID)
	--Make a copy
	t = Recount2:GetTable()
	for k, v in pairs(listValues[listID]) do
		t[k] = v
	end
	return t
end

function Recount2_Widgets:SetListCheckValue(listID, listEntries)
	for _, checkbox in ipairs(listFrames[listID]) do
		if listEntries[checkbox.listValue] then
			checkbox:SetValue(true)
		else
			checkbox:SetValue(false)
		end
	end
end

function Recount2_Widgets:SetCheckUpdateFunc(id, func, args)
	updateFuncs[id] = {func, args}
end
